/*
 *  ACPISensors.h
 *  HWSensors
 *
 *  Created by kozlek on 12/11/10.
 *  Copyright 2010 Slice. All rights reserved.
 *
 */

#ifndef __HWSensors__ACPISensors__
#define __HWSensors__ACPISensors__

#include "FakeSMCPlugin.h"

#include "IOKit/acpi/IOACPIPlatformDevice.h"

#if DEBUG
#define kACPISensorsDebug   0
#else
#define kACPISensorsDebug   0
#endif

#define ACPISensorsDebugLog(string, args...)	do { if (kACPISensorsDebug) { IOLog ("%s (%s): [Debug] " string "\n",getName(), acpiDevice->getName() , ## args); } } while(0)
#define ACPISensorsWarningLog(string, args...) do { IOLog ("%s (%s): [Warning] " string "\n",getName(), acpiDevice->getName(), ## args); } while(0)
#define ACPISensorsErrorLog(string, args...) do { IOLog ("%s (%s): [Error] " string "\n",getName(), acpiDevice->getName() , ## args); } while(0)
#define ACPISensorsFatalLog(string, args...) do { IOLog ("%s (%s): [Fatal] " string "\n",getName(), acpiDevice->getName() , ## args); } while(0)
#define ACPISensorsInfoLog(string, args...)	do { IOLog ("%s (%s): " string "\n",getName(), acpiDevice->getName() , ## args); } while(0)

class EXPORT ACPISensors : public FakeSMCPlugin
{
    OSDeclareDefaultStructors(ACPISensors)
    
private:
	IOACPIPlatformDevice    *acpiDevice;
    OSArray                 *methods;
    bool                    useKelvins;
    
    void                    addSensorsFromDictionary(OSDictionary *dictionary, FakeSMCSensorCategory category);
    void                    addSensorsFromArray(OSArray *array, FakeSMCSensorCategory category);
    
protected:
    virtual bool           willReadSensorValue(FakeSMCSensor *sensor, float *outValue);
    
public:
    virtual bool			start(IOService *provider);
};

#endif /* defined(__HWSensors__ACPISensors__) */
